module Simple::Sharding
  module ActiveRecordExtensions
    def self.extend_active_record_scope
      # return if exist
      return if ActiveRecord::Base.respond_to?(:sharding)

      # include sharding in class and instance
      ActiveRecord::Base.extend ScopeMethods
      ActiveRecord::Base.include ScopeMethods
      ActiveRecord::extend CaseFixer

      # Includes scope method in scopes
      ActiveRecord::Relation.include ScopeMethods
      ActiveRecord::Relation.extend CaseFixer

      # Includes scope method in has_many and habtm relations
      ActiveRecord::Scoping.include ScopeMethods
      ActiveRecord::Scoping.extend CaseFixer
    end
  end

  module ScopeMethods
    def sharding(shard_id)
      raise 'Error for sharding block.' if block_given?


    end
  end

  # Return value of the #sharding scope method. Allows us to chain the shard
  # choice with other ActiveRecord scopes
  class ScopeProxy
    attr_accessor :original_scope

    def initialize(shard_id, original_scope)
      @shard_id = shard_id
      @original_scope = original_scope
    end

    def sharding(shard_id)
      @shard_id = shard_id
      self
    end

    def method_missing(method, *args, &block)
      res = Core.sharding(@shard_id) do
        @original_scope.send(method, *args, &block)
      end

      # if result is still a scope (responds to to_sql), update original scope
      # and return proxy to continue chaining
      if res.respond_to?(:to_sql)
        @original_scope = res
        return self
      end

      # return res
      res
    end

    # Delegates == to method_missing so that User.sharding(:id).where(:name => "Mike")
    # gets run in the correct shard context when #== is evaluated.
    def ==(other)
      method_missing(:==, other)
    end
    alias_method :eql?, :==
  end
end
