require 'active_support/per_thread_registry'

module Simple::Sharding
  class ShardThreadRegistry
    extend ActiveSupport::PerThreadRegistry
    attr_accessor :_current_shard_id

    # Creates two thread-specific stacks to store the shard of connection
    # The top of the stack indicates the current connection
    # This allows us to have nested blocks of #using_shard and keep track of the
    # connections as we open/close those blocks
    thread_mattr_accessor :_shard_id_stack
    # auxiliary stack that keeps track of wether each shard connection was used
    # inside its respective using_shard block (so we can print an alert if not)
    thread_mattr_accessor :_shard_connection_used_stack

    def self.shard_id_stack; self._shard_id_stack ||= [] end;
    def self.shard_connection_used_stack; self._shard_connection_used_stack ||= [] end;


    # Returns the current shard id (for the current Thread)
    def self.current_shard_id
      shard_id_stack.last
    end


    def self.connect_to_master?
      _current_shard_id.nil?
    end

    def self.current_shard_id
      _current_shard_id
    end

    def self.current_shard_id=(shard_id)
      self._current_shard_id = shard_id.blank? ? nil : shard_id.to_sym
    end

    # adds shard connection to the stack
    def self.push_current_shard(shard_id)
      # this line supresses the unused connection warning when there are nested
      # using_shard blocks. We suppress the warning because we view nested using_shard
      # blocks as a override
      notify_connection_retrieved
      shard_id_stack.push(shard_id.blank? ? nil : shard_id.to_sym)
      shard_connection_used_stack.push(false)
    end


    # notifies the current connection was used (wee keep track of this to warn
    # the user in case the connection is not used)
    def self.notify_connection_retrieved
      shard_connection_used_stack[-1] = true if shard_connection_used_stack.present?
    end

    # removes shard connection to the stack
    def self.pop_current_shard
      [shard_id_stack.pop, shard_connection_used_stack.pop]
    end
  end
end
